@@ -4,7 +4,8 @@
 Created on Mon Jul  9 14:14:29 2018
 """
 import pytest
-from pyaerocom.conftest import lustre_unavail
+from pyaerocom import const
+from pyaerocom.conftest import does_not_raise_exception
 from pyaerocom.io import ReadUngridded
 
 
@@ -44,24 +45,63 @@ def test_supported():
                                                  'GHOST.EBAS.hourly',
                                                  'GHOST.EBAS.daily'])
 
-@lustre_unavail
-def test_basic_attributes():
-    r = ReadUngridded('AeronetSunV3Lev2.daily')
-    assert not r.ignore_cache
-    assert r.datasets_to_read == ['AeronetSunV3Lev2.daily']
-    assert type(r.get_reader()).__name__ == 'ReadAeronetSunV3'
-    assert r.dataset_provides_variables() == ['od340aer', 'od440aer',
-                                              'od500aer', 'od870aer',
-                                              'ang4487aer', 'ang44&87aer',
-                                              'od550aer']
 
-@lustre_unavail
-def test_read_aeronet_sunv3():
-    r = ReadUngridded()
-    data = r.read('AeronetSunV3Lev2.daily',
-                   vars_to_retrieve=['od550aer', 'ang4487aer'],
-                   file_pattern='Bo*')
-    data._check_index()
+@pytest.mark.parametrize('dsr,vtr,oc,fp,kw,nst,nmeta,raises,caching', [
+    ('AeronetSunV3L2Subset.daily', 'od550aer',False,None,{},22,22,
+     does_not_raise_exception(),False),
+    ('AeronetSunV3L2Subset.daily', 'od550aer', False,
+     dict(station_name='La_Paz'),{},1, 1,
+     does_not_raise_exception(),True),
+    ('AeronetSunV3L2Subset.daily', 'od550aer', False,
+     dict(station_name=['La_Paz', 'AAO*']),{},2, 2,
+     does_not_raise_exception(),True),
+    ('AeronetSunV3L2Subset.daily', ['od550aer'], False,
+     dict(altitude=[1000, 10000],
+          ignore_station_names=dict(od550aer='La_Paz')),{},2, 2,
+     does_not_raise_exception(),True),
+    ('AeronetSunV3L2Subset.daily', ['od550aer', 'ang4487aer'], False,
+     dict(altitude=[1000, 10000],
+          ignore_station_names=dict(od550aer='La_Paz')),{},2, 2,
+     pytest.raises(NotImplementedError),True),
+    ('AeronetSunV3L2Subset.daily', 'od550aer',True,None,{},22,22,
+     does_not_raise_exception(),True),
+    ('AeronetSunV3L2Subset.daily', 'od550aer',True,
+     dict(altitude=[1000, 10000]),{},3,3,
+     does_not_raise_exception(),True),
+    ('AeronetSunV3L2Subset.daily', 'od550aer', False,
+     dict(altitude=[1000, 10000],
+          ignore_station_names='La_*'),{},2, 2,
+     does_not_raise_exception(),True),
+    ('AeronetSunV3L2Subset.daily', 'od550aer', False,
+     dict(altitude=[1000, 10000],
+          ignore_station_names=['La_*', 'Mauna_Loa']),{},1, 1,
+     does_not_raise_exception(),True)
+
+    ])
+def test_ReadUngridded_read(dsr,vtr,oc,fp,kw,nst,nmeta,raises,caching):
+    reader = ReadUngridded()
+
+    const.CACHING = False if not caching else True
+    with raises:
+        data = reader.read(datasets_to_read=dsr,
+                           vars_to_retrieve=vtr,
+                           only_cached=oc,
+                           filter_post=fp,
+                           **kw)
+
+        assert len(data.metadata) == nmeta
+        assert len(data.unique_station_names) == nst
+
+    const.CACHING = True
+
+def test_basic_attributes():
+    reader = ReadUngridded()
+    assert not reader.ignore_cache
+    assert reader.datasets_to_read == []
+    with pytest.raises(ValueError):
+        reader.get_reader()
+    with pytest.raises(ValueError):
+        reader.dataset_provides_variables()
 
 if __name__=="__main__":
     import sys
