@@ -6,20 +6,92 @@ Created on Mon Nov 25 15:27:28 2019
 @author: jonasg
 """
 import pytest
-import os
-from pyaerocom import conftest
+import os, tempfile
+from pyaerocom.conftest import does_not_raise_exception, PYADIR
 from pyaerocom import const as DEFAULT_CFG
+import pyaerocom.config as testmod
 import getpass
-
 USER = getpass.getuser()
 
-import pyaerocom.config as testmod
+_TEMPDIR = tempfile.mkdtemp()
+CFG_FILE_WRONG = os.path.join(_TEMPDIR, 'paths.txt')
+
+LOCAL_DB_DIR = os.path.join(_TEMPDIR, 'data')
+os.makedirs(os.path.join(LOCAL_DB_DIR, 'modeldata'))
+os.makedirs(os.path.join(LOCAL_DB_DIR, 'obsdata'))
+open(CFG_FILE_WRONG, 'w').close()
+
+CFG_FILE = os.path.join(PYADIR, 'data/paths.ini')
+
+def test_CFG_FILE_EXISTS():
+    assert os.path.exists(CFG_FILE)
+
+def test_CFG_FILE_WRONG_EXISTS():
+    assert os.path.exists(CFG_FILE_WRONG)
+
+def test_LOCAL_DB_DIR_EXISTS():
+    assert os.path.exists(LOCAL_DB_DIR)
 
 @pytest.fixture(scope='module')
 def empty_cfg():
     cfg = testmod.Config(try_infer_environment=False)
     return cfg
 
+def test_Config_ALL_DATABASE_IDS(empty_cfg):
+    assert empty_cfg.ALL_DATABASE_IDS == ['metno', 'users-db', 'local-db']
+
+@pytest.mark.parametrize('config_file,try_infer_environment,raises', [
+    (CFG_FILE_WRONG, False, pytest.raises(ValueError)),
+    (f'/home/{USER}/blaaaa.ini', False, pytest.raises(FileNotFoundError)),
+    (None, False, does_not_raise_exception()),
+    (None, True, does_not_raise_exception()),
+    (CFG_FILE, False, does_not_raise_exception())
+    ])
+def test_Config___init__(config_file,try_infer_environment,raises):
+    with raises:
+        cfg = testmod.Config(config_file, try_infer_environment)
+
+@pytest.mark.parametrize('basedir,raises,envid', [
+    ('/blaaa', pytest.raises(FileNotFoundError), None),
+    (LOCAL_DB_DIR, does_not_raise_exception(), 'local-db'),
+    ])
+def test_Config__infer_config_from_basedir(basedir,raises, envid):
+    cfg = testmod.Config(try_infer_environment=False)
+    with raises:
+        res = cfg._infer_config_from_basedir(basedir)
+        assert res[1] == envid
+
+def test_Config_has_access_lustre():
+    cfg = testmod.Config(try_infer_environment=False)
+    assert not cfg.has_access_lustre
+
+def test_Config_has_access_users_database():
+    cfg = testmod.Config(try_infer_environment=False)
+    assert not cfg.has_access_users_database
+
+@pytest.mark.parametrize('cfg_id, basedir, init_obslocs_ungridded,init_data_search_dirs, raises, num_obs, num_dirs', [
+    ('metno', None,False,False,does_not_raise_exception(),0,0),
+    ('metno', None,True,False,does_not_raise_exception(),0,0),
+    ('metno', None,True,True,does_not_raise_exception(),0,0),
+    ('metno',f'/home/{USER}',True,True,does_not_raise_exception(),0,0),
+    ('users-db', None,False,False,does_not_raise_exception(),0,0),
+    ])
+def test_Config_read_config(cfg_id, basedir, init_obslocs_ungridded,
+                            init_data_search_dirs, raises, num_obs,
+                            num_dirs):
+    cfg = testmod.Config(try_infer_environment=False)
+    cfg_file = cfg._config_files[cfg_id]
+    assert os.path.exists(cfg_file)
+    with raises:
+        cfg.read_config(cfg_file, basedir, init_obslocs_ungridded,
+                        init_data_search_dirs)
+        assert len(cfg.DATA_SEARCH_DIRS) == num_dirs
+        assert len(cfg.OBSLOCS_UNGRIDDED) == num_obs
+        assert os.path.exists(cfg.OUTPUTDIR)
+        assert os.path.exists(cfg.COLOCATEDDATADIR)
+        assert os.path.exists(cfg.CACHEDIR)
+
+
 def test_empty_class_header(empty_cfg):
     cfg = empty_cfg
     assert cfg.AERONET_SUN_V2L15_AOD_DAILY_NAME == 'AeronetSunV2Lev1.5.daily'
@@ -77,10 +149,6 @@ def test_empty_class_header(empty_cfg):
     assert cfg.CLIM_MIN_COUNT == dict(daily = 30,
                                      monthly = 5)
 
-    #names of the different obs networks
-    assert cfg.OBSNET_NONE == 'NONE'
-    assert cfg.NOMODELNAME == 'OBSERVATIONS-ONLY'
-
     # names for the satellite data sets
     assert cfg.SENTINEL5P_NAME == 'Sentinel5P'
     assert cfg.AEOLUS_NAME == 'AeolusL2A'
@@ -110,25 +178,21 @@ def test_empty_class_header(empty_cfg):
     config_files = {
         'lustre'        : os.path.join(__dir__, 'data', 'paths.ini'),
         'user_server'   : os.path.join(__dir__, 'data', 'paths_user_server.ini'),
-        'testdata'      : os.path.join(__dir__, 'data', 'paths_testdata.ini'),
         'local-db'      : os.path.join(__dir__, 'data', 'paths_local_database.ini')
         }
 
     assert cfg._config_ini_lustre == config_files['lustre']
     assert cfg._config_ini_user_server == config_files['user_server']
-    assert cfg._config_ini_testdata == config_files['testdata']
     assert cfg._config_ini_localdb == config_files['local-db']
 
     assert cfg._config_files == {
             'metno'            : config_files['lustre'],
             'users-db'         : config_files['user_server'],
-            'testdata'         : config_files['testdata'],
             'local-db'         : config_files['local-db']
     }
     assert cfg._check_subdirs_cfg == {
             'metno'       : 'aerocom',
             'users-db'    : 'AMAP',
-            'testdata'    : 'modeldata',
             'local-db'    : 'modeldata'
     }
 
@@ -136,8 +200,6 @@ def test_empty_class_header(empty_cfg):
     assert cfg._coords_info_file == os.path.join(__dir__, 'data', 'coords.ini')
     dbdirs = {'lustre/storeA/project': 'metno',
               'metno/aerocom_users_database': 'users-db',
-              'pyaerocom-testdata/': 'testdata',
-              'MyPyaerocom/pyaerocom-testdata': 'testdata',
               'MyPyaerocom/data': 'local-db'}
     for sd, name in dbdirs.items():
         assert sd in cfg._DB_SEARCH_SUBDIRS
@@ -194,7 +256,6 @@ def test_default_config():
     assert cfg._outputdir == mypydir
     assert cfg.OUTPUTDIR == mypydir
 
-    assert cfg._cache_basedir == mkpath(mypydir, '_cache')
     assert cfg.CACHEDIR == mkpath(mypydir, f'_cache/{USER}')
 
     check = mkpath(mypydir, 'colocated_data')
