@@ -5,7 +5,7 @@ from ast import literal_eval
 import re, fnmatch
 from cf_units import Unit
 from configparser import ConfigParser
-from pyaerocom import logger, print_log
+from pyaerocom import logger, print_log, var_groups
 from pyaerocom.obs_io import OBS_WAVELENGTH_TOL_NM
 from pyaerocom.exceptions import VariableDefinitionError
 from pyaerocom._lowlevel_helpers import list_to_shortstr, dict_to_str
@@ -29,10 +29,11 @@ class VarNameInfo(object):
         'ang*'  :   'Column',
         'load*' :   'Column',
         'wet*'  :   'Surface',
-        'dry*'  :   'Surface'}
+        'dry*'  :   'Surface',
+        'emi*'  :   'Surface'}
 
     def __init__(self, var_name):
-        self.var_name = var_name.lower()
+        self.var_name = var_name
         self._nums = []
         try:
             self._nums = self._numbers_in_string(var_name)
@@ -102,13 +103,6 @@ class VarNameInfo(object):
                                           'from variable name')
         return self._nums[0]
 
-    @property
-    def is_optical_density(self):
-        """Boolean specifying whether variable is an optical depth"""
-        if re.match(self.PATTERNS['od'], self.var_name) and self.contains_wavelength_nm:
-            return True
-        return False
-
     def in_wavelength_range(self, low, high):
         """Boolean specifying whether variable is within wavelength range
 
@@ -322,6 +316,8 @@ class Variable(object):
         scatter plot on loglog scale
     scat_scale_factor : float
         scale factor for scatter plot
+    map_cmap : str
+        name of default colormap (matplotlib) of this variable.
     map_vmin : float
         data value corresponding to lower end of colormap in map plots of this
         quantity
@@ -354,6 +350,7 @@ class Variable(object):
         'scat_loglog': str2bool,
         'scat_scale_factor': float,
         'dry_rh_max':float,
+        'map_cmap' : str,
         'map_vmin': float,
         'map_vmax': float,
         'map_cbar_levels': literal_eval_list,
@@ -363,14 +360,13 @@ class Variable(object):
     #maybe used in config
     ALT_NAMES = {'unit' : 'units'}
 
-    RH_MAX_DRY = 0.4
-
     plot_info_keys = ['scat_xlim',
                       'scat_ylim',
                       'scat_loglog',
                       'scat_scale_factor',
                       'map_vmin',
                       'map_vmax',
+                      'map_cmap',
                       'map_c_under',
                       'map_c_over',
                       'map_cbar_levels',
@@ -378,7 +374,6 @@ class Variable(object):
 
     @staticmethod
     def _check_input_var_name(var_name):
-        var_name = var_name.lower()
         if '3d' in var_name:
             var_name = var_name.replace('3d','')
         return var_name
@@ -387,7 +382,6 @@ class Variable(object):
         # save orig. input for whatever reasons
         self._var_name_input = var_name
 
-        # var_name in lowercase and cleaned up (3d and dry removed if applicable)
         self.var_name = var_name = self._check_input_var_name(var_name)
         self._var_name_aerocom = None
 
@@ -433,27 +427,110 @@ class Variable(object):
 
     @property
     def var_name_aerocom(self):
+        """AeroCom variable name of the input variable"""
         vna = self._var_name_aerocom
         return self.var_name if vna is None else vna
 
     @property
     def var_name_input(self):
-        """Input variable (in lowercase)"""
-        return self._var_name_input.lower()
+        """Input variable"""
+        return self._var_name_input
 
     @property
     def is_3d(self):
         """True if str '3d' is contained in :attr:`var_name_input`"""
-        return True if '3d' in self.var_name_input else False
+        return True if '3d' in self.var_name_input.lower() else False
 
     @property
     def is_wavelength_dependent(self):
+        """Indicates whether this variable is wavelength dependent"""
         return True if self.wavelength_nm is not None else False
 
     @property
-    def is_dry(self):
-        """True if str 'dry' is contained in :attr:`var_name_input`"""
-        return True if 'dry' in self.var_name_input else False
+    def is_at_dry_conditions(self):
+        """Indicate whether variable denotes dry conditions"""
+        var_name = self.var_name_aerocom
+        if var_name.startswith('dry'): # dry deposition
+            return False
+        return True if 'dry' in var_name else False
+
+    @property
+    def is_deposition(self):
+        """
+        Indicates whether input variables is a deposition rate
+
+        Note
+        ----
+        This funtion only identifies wet and dry deposition based on the variable
+        names, there might be other variables that are deposition variables but
+        cannot be identified by this function.
+
+        Parameters
+        ----------
+        var_name : str
+            Name of variable to be checked
+
+        Returns
+        -------
+        bool
+            If True, then variable name denotes a deposition variables
+
+        """
+        var_name = self.var_name_aerocom
+        if var_name.startswith(var_groups.drydep_startswith):
+            return True
+        elif var_name.startswith(var_groups.wetdep_startswith):
+            return True
+        elif var_name in var_groups.dep_add_vars:
+            return True
+        return False
+
+    @property
+    def is_emission(self):
+        """
+        Indicates whether input variables is an emission rate
+
+        Note
+        ----
+        This funtion only identifies wet and dry deposition based on the variable
+        names, there might be other variables that are deposition variables but
+        cannot be identified by this function.
+
+        Parameters
+        ----------
+        var_name : str
+            Name of variable to be checked
+
+        Returns
+        -------
+        bool
+            If True, then variable name denotes a deposition variables
+
+        """
+        var_name = self.var_name_aerocom
+        if var_name.startswith(var_groups.emi_startswith):
+            return True
+        elif var_name in var_groups.emi_add_vars:
+            return True
+        return False
+
+    @property
+    def is_rate(self):
+        """Indicates whether variable name is a rate
+
+        Rates include e.g. deposition or emission rate variables
+
+        Returns
+        -------
+        bool
+        """
+        if self.is_emission:
+            return True
+        elif self.is_deposition:
+            return True
+        elif self.var_name_aerocom in var_groups.rate_add_vars:
+            return True
+        return False
 
     @property
     def is_alias(self):
@@ -511,8 +588,10 @@ class Variable(object):
 
     @property
     def var_name_info(self):
+
         return VarNameInfo(self.var_name)
 
+
     @property
     def aliases(self):
         """Alias variable names that are frequently found or used
@@ -569,11 +648,6 @@ class Variable(object):
         -------
         bool
             True, if default could be loaded, False if not
-
-        Raises
-        ------
-        IOError
-            if regions.ini file does not exist
         """
         if cfg is None:
             cfg = self.read_config()
@@ -692,7 +766,7 @@ class VarCollection(object):
         aliases.ini.
         """
         if self._all_vars is None:
-            all_vars = [k.lower() for k in self._cfg_parser.keys()]
+            all_vars = [k for k in self._cfg_parser.keys()]
             all_vars.extend(self._vars_added.keys())
             #all_vars.extend(list(_read_alias_ini()))
             self._all_vars = all_vars
@@ -885,3 +959,6 @@ def all_var_names():
     """Helper method that returns all currently defined variable names"""
     return [k for k in Variable.read_config().keys()]
 
+if __name__=='__main__':
+    var = Variable('od550lt1aer')
+    print(var)
\ No newline at end of file
