@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 from .aerocom_evaluation import AerocomEvaluation
-from .trends_evaluation import TrendsEvaluation
+#from .trends_evaluation import TrendsEvaluation
 from .helpers import ModelConfigEval, ObsConfigEval
 from .web_naming_conventions import VAR_MAPPING
 from .utils import *
